package org.eclipse.jdt.internal.compiler.ast;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import org.eclipse.jdt.internal.compiler.ASTVisitor;
import org.eclipse.jdt.internal.compiler.ast.ASTNode;
import org.eclipse.jdt.internal.compiler.ast.CastExpression;
import org.eclipse.jdt.internal.compiler.ast.Expression;
import org.eclipse.jdt.internal.compiler.ast.ExpressionContext;
import org.eclipse.jdt.internal.compiler.ast.IPolyExpression;
import org.eclipse.jdt.internal.compiler.ast.NullAnnotationMatching;
import org.eclipse.jdt.internal.compiler.ast.OperatorExpression;
import org.eclipse.jdt.internal.compiler.codegen.BranchLabel;
import org.eclipse.jdt.internal.compiler.codegen.CodeStream;
import org.eclipse.jdt.internal.compiler.flow.FlowContext;
import org.eclipse.jdt.internal.compiler.flow.FlowInfo;
import org.eclipse.jdt.internal.compiler.flow.UnconditionalFlowInfo;
import org.eclipse.jdt.internal.compiler.impl.CompilerOptions;
import org.eclipse.jdt.internal.compiler.impl.Constant;
import org.eclipse.jdt.internal.compiler.lookup.BaseTypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.lookup.LookupEnvironment;
import org.eclipse.jdt.internal.compiler.lookup.MethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.PolyTypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.Scope;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;

public class ConditionalExpression extends OperatorExpression implements IPolyExpression {
  public Expression condition;
  
  public Expression valueIfTrue;
  
  public Expression valueIfFalse;
  
  public Constant optimizedBooleanConstant;
  
  public Constant optimizedIfTrueConstant;
  
  public Constant optimizedIfFalseConstant;
  
  int trueInitStateIndex = -1;
  
  int falseInitStateIndex = -1;
  
  int mergedInitStateIndex = -1;
  
  private int nullStatus = 1;
  
  int ifFalseNullStatus;
  
  int ifTrueNullStatus;
  
  private TypeBinding expectedType;
  
  private ExpressionContext expressionContext = ExpressionContext.VANILLA_CONTEXT;
  
  private boolean isPolyExpression = false;
  
  private TypeBinding originalValueIfTrueType;
  
  private TypeBinding originalValueIfFalseType;
  
  private boolean use18specifics;
  
  public ConditionalExpression(Expression condition, Expression valueIfTrue, Expression valueIfFalse) {
    this.condition = condition;
    this.valueIfTrue = valueIfTrue;
    this.valueIfFalse = valueIfFalse;
    this.sourceStart = condition.sourceStart;
    this.sourceEnd = valueIfFalse.sourceEnd;
  }
  
  public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, FlowInfo flowInfo) {
    FlowInfo mergedInfo;
    int initialComplaintLevel = ((flowInfo.reachMode() & 0x3) != 0) ? 1 : 0;
    Constant cst = this.condition.optimizedBooleanConstant();
    boolean isConditionOptimizedTrue = (cst != Constant.NotAConstant && cst.booleanValue());
    boolean isConditionOptimizedFalse = (cst != Constant.NotAConstant && !cst.booleanValue());
    int mode = flowInfo.reachMode();
    flowInfo = this.condition.analyseCode(currentScope, flowContext, flowInfo, (cst == Constant.NotAConstant));
    flowContext.conditionalLevel++;
    FlowInfo trueFlowInfo = flowInfo.initsWhenTrue().copy();
    CompilerOptions compilerOptions = currentScope.compilerOptions();
    if (isConditionOptimizedFalse) {
      if ((mode & 0x3) == 0)
        trueFlowInfo.setReachMode(1); 
      if (!isKnowDeadCodePattern(this.condition) || compilerOptions.reportDeadCodeInTrivialIfStatement)
        this.valueIfTrue.complainIfUnreachable(trueFlowInfo, currentScope, initialComplaintLevel, false); 
    } 
    this.trueInitStateIndex = currentScope.methodScope().recordInitializationStates(trueFlowInfo);
    trueFlowInfo = this.valueIfTrue.analyseCode(currentScope, flowContext, trueFlowInfo);
    this.valueIfTrue.checkNPEbyUnboxing(currentScope, flowContext, trueFlowInfo);
    this.ifTrueNullStatus = -1;
    if (compilerOptions.enableSyntacticNullAnalysisForFields) {
      this.ifTrueNullStatus = this.valueIfTrue.nullStatus(trueFlowInfo, flowContext);
      flowContext.expireNullCheckedFieldInfo();
    } 
    FlowInfo falseFlowInfo = flowInfo.initsWhenFalse().copy();
    if (isConditionOptimizedTrue) {
      if ((mode & 0x3) == 0)
        falseFlowInfo.setReachMode(1); 
      if (!isKnowDeadCodePattern(this.condition) || compilerOptions.reportDeadCodeInTrivialIfStatement)
        this.valueIfFalse.complainIfUnreachable(falseFlowInfo, currentScope, initialComplaintLevel, true); 
    } 
    this.falseInitStateIndex = currentScope.methodScope().recordInitializationStates(falseFlowInfo);
    falseFlowInfo = this.valueIfFalse.analyseCode(currentScope, flowContext, falseFlowInfo);
    this.valueIfFalse.checkNPEbyUnboxing(currentScope, flowContext, falseFlowInfo);
    flowContext.conditionalLevel--;
    if (isConditionOptimizedTrue) {
      mergedInfo = trueFlowInfo.addPotentialInitializationsFrom(falseFlowInfo);
      if (this.ifTrueNullStatus != -1) {
        this.nullStatus = this.ifTrueNullStatus;
      } else {
        this.nullStatus = this.valueIfTrue.nullStatus(trueFlowInfo, flowContext);
      } 
    } else if (isConditionOptimizedFalse) {
      mergedInfo = falseFlowInfo.addPotentialInitializationsFrom(trueFlowInfo);
      this.nullStatus = this.valueIfFalse.nullStatus(falseFlowInfo, flowContext);
    } else {
      computeNullStatus(trueFlowInfo, falseFlowInfo, flowContext);
      cst = this.optimizedIfTrueConstant;
      boolean isValueIfTrueOptimizedTrue = (cst != null && cst != Constant.NotAConstant && cst.booleanValue());
      boolean isValueIfTrueOptimizedFalse = (cst != null && cst != Constant.NotAConstant && !cst.booleanValue());
      cst = this.optimizedIfFalseConstant;
      boolean isValueIfFalseOptimizedTrue = (cst != null && cst != Constant.NotAConstant && cst.booleanValue());
      boolean isValueIfFalseOptimizedFalse = (cst != null && cst != Constant.NotAConstant && !cst.booleanValue());
      UnconditionalFlowInfo trueFlowTowardsTrue = trueFlowInfo.initsWhenTrue().unconditionalCopy();
      UnconditionalFlowInfo falseFlowTowardsTrue = falseFlowInfo.initsWhenTrue().unconditionalCopy();
      UnconditionalFlowInfo trueFlowTowardsFalse = trueFlowInfo.initsWhenFalse().unconditionalInits();
      UnconditionalFlowInfo falseFlowTowardsFalse = falseFlowInfo.initsWhenFalse().unconditionalInits();
      if (isValueIfTrueOptimizedFalse)
        trueFlowTowardsTrue.setReachMode(1); 
      if (isValueIfFalseOptimizedFalse)
        falseFlowTowardsTrue.setReachMode(1); 
      if (isValueIfTrueOptimizedTrue)
        trueFlowTowardsFalse.setReachMode(1); 
      if (isValueIfFalseOptimizedTrue)
        falseFlowTowardsFalse.setReachMode(1); 
      mergedInfo = 
        FlowInfo.conditional(
          (FlowInfo)trueFlowTowardsTrue.mergedWith(falseFlowTowardsTrue), 
          (FlowInfo)trueFlowTowardsFalse.mergedWith(falseFlowTowardsFalse));
    } 
    this.mergedInitStateIndex = 
      currentScope.methodScope().recordInitializationStates(mergedInfo);
    mergedInfo.setReachMode(mode);
    return mergedInfo;
  }
  
  public boolean checkNPE(BlockScope scope, FlowContext flowContext, FlowInfo flowInfo, int ttlForFieldCheck) {
    if ((this.nullStatus & 0x2) != 0) {
      scope.problemReporter().expressionNullReference((ASTNode)this);
    } else if ((this.nullStatus & 0x10) != 0) {
      scope.problemReporter().expressionPotentialNullReference((ASTNode)this);
    } 
    return true;
  }
  
  private void computeNullStatus(FlowInfo trueBranchInfo, FlowInfo falseBranchInfo, FlowContext flowContext) {
    if (this.ifTrueNullStatus == -1)
      this.ifTrueNullStatus = this.valueIfTrue.nullStatus(trueBranchInfo, flowContext); 
    this.ifFalseNullStatus = this.valueIfFalse.nullStatus(falseBranchInfo, flowContext);
    if (this.ifTrueNullStatus == this.ifFalseNullStatus) {
      this.nullStatus = this.ifTrueNullStatus;
      return;
    } 
    if (trueBranchInfo.reachMode() != 0) {
      this.nullStatus = this.ifFalseNullStatus;
      return;
    } 
    if (falseBranchInfo.reachMode() != 0) {
      this.nullStatus = this.ifTrueNullStatus;
      return;
    } 
    int status = 0;
    int combinedStatus = this.ifTrueNullStatus | this.ifFalseNullStatus;
    if ((combinedStatus & 0x12) != 0)
      status |= 0x10; 
    if ((combinedStatus & 0x24) != 0)
      status |= 0x20; 
    if ((combinedStatus & 0x9) != 0)
      status |= 0x8; 
    if (status > 0)
      this.nullStatus = status; 
  }
  
  public void generateCode(BlockScope currentScope, CodeStream codeStream, boolean valueRequired) {
    int pc = codeStream.position;
    if (this.constant != Constant.NotAConstant) {
      if (valueRequired)
        codeStream.generateConstant(this.constant, this.implicitConversion); 
      codeStream.recordPositionsFrom(pc, this.sourceStart);
      return;
    } 
    Constant cst = this.condition.optimizedBooleanConstant();
    boolean needTruePart = !(cst != Constant.NotAConstant && !cst.booleanValue());
    boolean needFalsePart = !(cst != Constant.NotAConstant && cst.booleanValue());
    BranchLabel endifLabel = new BranchLabel(codeStream);
    BranchLabel falseLabel = new BranchLabel(codeStream);
    falseLabel.tagBits |= 0x2;
    this.condition.generateOptimizedBoolean(
        currentScope, 
        codeStream, 
        null, 
        falseLabel, 
        (cst == Constant.NotAConstant));
    if (this.trueInitStateIndex != -1) {
      codeStream.removeNotDefinitelyAssignedVariables(
          (Scope)currentScope, 
          this.trueInitStateIndex);
      codeStream.addDefinitelyAssignedVariables((Scope)currentScope, this.trueInitStateIndex);
    } 
    if (needTruePart) {
      this.valueIfTrue.generateCode(currentScope, codeStream, valueRequired);
      if (needFalsePart) {
        int position = codeStream.position;
        codeStream.goto_(endifLabel);
        codeStream.recordPositionsFrom(position, this.valueIfTrue.sourceEnd);
        if (valueRequired)
          switch (this.resolvedType.id) {
            case 7:
            case 8:
              codeStream.decrStackSize(2);
              break;
            default:
              codeStream.decrStackSize(1);
              break;
          }  
      } 
    } 
    if (needFalsePart) {
      if (this.falseInitStateIndex != -1) {
        codeStream.removeNotDefinitelyAssignedVariables(
            (Scope)currentScope, 
            this.falseInitStateIndex);
        codeStream.addDefinitelyAssignedVariables((Scope)currentScope, this.falseInitStateIndex);
      } 
      if (falseLabel.forwardReferenceCount() > 0)
        falseLabel.place(); 
      this.valueIfFalse.generateCode(currentScope, codeStream, valueRequired);
      if (valueRequired)
        codeStream.recordExpressionType(this.resolvedType); 
      if (needTruePart)
        endifLabel.place(); 
    } 
    if (this.mergedInitStateIndex != -1)
      codeStream.removeNotDefinitelyAssignedVariables(
          (Scope)currentScope, 
          this.mergedInitStateIndex); 
    if (valueRequired)
      codeStream.generateImplicitConversion(this.implicitConversion); 
    codeStream.recordPositionsFrom(pc, this.sourceStart);
  }
  
  public void generateOptimizedBoolean(BlockScope currentScope, CodeStream codeStream, BranchLabel trueLabel, BranchLabel falseLabel, boolean valueRequired) {
    // Byte code:
    //   0: aload_2
    //   1: getfield position : I
    //   4: istore #6
    //   6: aload_0
    //   7: getfield constant : Lorg/eclipse/jdt/internal/compiler/impl/Constant;
    //   10: getstatic org/eclipse/jdt/internal/compiler/impl/Constant.NotAConstant : Lorg/eclipse/jdt/internal/compiler/impl/Constant;
    //   13: if_acmpeq -> 27
    //   16: aload_0
    //   17: getfield constant : Lorg/eclipse/jdt/internal/compiler/impl/Constant;
    //   20: invokevirtual typeID : ()I
    //   23: iconst_5
    //   24: if_icmpeq -> 61
    //   27: aload_0
    //   28: getfield valueIfTrue : Lorg/eclipse/jdt/internal/compiler/ast/Expression;
    //   31: getfield implicitConversion : I
    //   34: sipush #255
    //   37: iand
    //   38: iconst_4
    //   39: ishr
    //   40: iconst_5
    //   41: if_icmpne -> 61
    //   44: aload_0
    //   45: getfield valueIfFalse : Lorg/eclipse/jdt/internal/compiler/ast/Expression;
    //   48: getfield implicitConversion : I
    //   51: sipush #255
    //   54: iand
    //   55: iconst_4
    //   56: ishr
    //   57: iconst_5
    //   58: if_icmpeq -> 73
    //   61: aload_0
    //   62: aload_1
    //   63: aload_2
    //   64: aload_3
    //   65: aload #4
    //   67: iload #5
    //   69: invokespecial generateOptimizedBoolean : (Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;Lorg/eclipse/jdt/internal/compiler/codegen/CodeStream;Lorg/eclipse/jdt/internal/compiler/codegen/BranchLabel;Lorg/eclipse/jdt/internal/compiler/codegen/BranchLabel;Z)V
    //   72: return
    //   73: aload_0
    //   74: getfield condition : Lorg/eclipse/jdt/internal/compiler/ast/Expression;
    //   77: getfield constant : Lorg/eclipse/jdt/internal/compiler/impl/Constant;
    //   80: astore #7
    //   82: aload_0
    //   83: getfield condition : Lorg/eclipse/jdt/internal/compiler/ast/Expression;
    //   86: invokevirtual optimizedBooleanConstant : ()Lorg/eclipse/jdt/internal/compiler/impl/Constant;
    //   89: astore #8
    //   91: aload #7
    //   93: getstatic org/eclipse/jdt/internal/compiler/impl/Constant.NotAConstant : Lorg/eclipse/jdt/internal/compiler/impl/Constant;
    //   96: if_acmpeq -> 107
    //   99: aload #7
    //   101: invokevirtual booleanValue : ()Z
    //   104: ifeq -> 123
    //   107: aload #8
    //   109: getstatic org/eclipse/jdt/internal/compiler/impl/Constant.NotAConstant : Lorg/eclipse/jdt/internal/compiler/impl/Constant;
    //   112: if_acmpeq -> 127
    //   115: aload #8
    //   117: invokevirtual booleanValue : ()Z
    //   120: ifne -> 127
    //   123: iconst_0
    //   124: goto -> 128
    //   127: iconst_1
    //   128: istore #9
    //   130: aload #7
    //   132: getstatic org/eclipse/jdt/internal/compiler/impl/Constant.NotAConstant : Lorg/eclipse/jdt/internal/compiler/impl/Constant;
    //   135: if_acmpeq -> 146
    //   138: aload #7
    //   140: invokevirtual booleanValue : ()Z
    //   143: ifne -> 162
    //   146: aload #8
    //   148: getstatic org/eclipse/jdt/internal/compiler/impl/Constant.NotAConstant : Lorg/eclipse/jdt/internal/compiler/impl/Constant;
    //   151: if_acmpeq -> 166
    //   154: aload #8
    //   156: invokevirtual booleanValue : ()Z
    //   159: ifeq -> 166
    //   162: iconst_0
    //   163: goto -> 167
    //   166: iconst_1
    //   167: istore #10
    //   169: new org/eclipse/jdt/internal/compiler/codegen/BranchLabel
    //   172: dup
    //   173: aload_2
    //   174: invokespecial <init> : (Lorg/eclipse/jdt/internal/compiler/codegen/CodeStream;)V
    //   177: astore #12
    //   179: aload #7
    //   181: getstatic org/eclipse/jdt/internal/compiler/impl/Constant.NotAConstant : Lorg/eclipse/jdt/internal/compiler/impl/Constant;
    //   184: if_acmpne -> 199
    //   187: aload #8
    //   189: getstatic org/eclipse/jdt/internal/compiler/impl/Constant.NotAConstant : Lorg/eclipse/jdt/internal/compiler/impl/Constant;
    //   192: if_acmpne -> 199
    //   195: iconst_1
    //   196: goto -> 200
    //   199: iconst_0
    //   200: istore #13
    //   202: aload_0
    //   203: getfield condition : Lorg/eclipse/jdt/internal/compiler/ast/Expression;
    //   206: aload_1
    //   207: aload_2
    //   208: aconst_null
    //   209: new org/eclipse/jdt/internal/compiler/codegen/BranchLabel
    //   212: dup
    //   213: aload_2
    //   214: invokespecial <init> : (Lorg/eclipse/jdt/internal/compiler/codegen/CodeStream;)V
    //   217: dup
    //   218: astore #11
    //   220: iload #13
    //   222: invokevirtual generateOptimizedBoolean : (Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;Lorg/eclipse/jdt/internal/compiler/codegen/CodeStream;Lorg/eclipse/jdt/internal/compiler/codegen/BranchLabel;Lorg/eclipse/jdt/internal/compiler/codegen/BranchLabel;Z)V
    //   225: aload_0
    //   226: getfield trueInitStateIndex : I
    //   229: iconst_m1
    //   230: if_icmpeq -> 251
    //   233: aload_2
    //   234: aload_1
    //   235: aload_0
    //   236: getfield trueInitStateIndex : I
    //   239: invokevirtual removeNotDefinitelyAssignedVariables : (Lorg/eclipse/jdt/internal/compiler/lookup/Scope;I)V
    //   242: aload_2
    //   243: aload_1
    //   244: aload_0
    //   245: getfield trueInitStateIndex : I
    //   248: invokevirtual addDefinitelyAssignedVariables : (Lorg/eclipse/jdt/internal/compiler/lookup/Scope;I)V
    //   251: iload #9
    //   253: ifeq -> 397
    //   256: aload_0
    //   257: getfield valueIfTrue : Lorg/eclipse/jdt/internal/compiler/ast/Expression;
    //   260: aload_1
    //   261: aload_2
    //   262: aload_3
    //   263: aload #4
    //   265: iload #5
    //   267: invokevirtual generateOptimizedBoolean : (Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;Lorg/eclipse/jdt/internal/compiler/codegen/CodeStream;Lorg/eclipse/jdt/internal/compiler/codegen/BranchLabel;Lorg/eclipse/jdt/internal/compiler/codegen/BranchLabel;Z)V
    //   270: iload #10
    //   272: ifeq -> 397
    //   275: aload #4
    //   277: ifnonnull -> 326
    //   280: aload_3
    //   281: ifnull -> 372
    //   284: aload_0
    //   285: getfield optimizedIfTrueConstant : Lorg/eclipse/jdt/internal/compiler/impl/Constant;
    //   288: astore #7
    //   290: aload #7
    //   292: ifnull -> 315
    //   295: aload #7
    //   297: getstatic org/eclipse/jdt/internal/compiler/impl/Constant.NotAConstant : Lorg/eclipse/jdt/internal/compiler/impl/Constant;
    //   300: if_acmpeq -> 315
    //   303: aload #7
    //   305: invokevirtual booleanValue : ()Z
    //   308: ifeq -> 315
    //   311: iconst_1
    //   312: goto -> 316
    //   315: iconst_0
    //   316: istore #14
    //   318: iload #14
    //   320: ifeq -> 372
    //   323: goto -> 397
    //   326: aload_3
    //   327: ifnonnull -> 372
    //   330: aload_0
    //   331: getfield optimizedIfTrueConstant : Lorg/eclipse/jdt/internal/compiler/impl/Constant;
    //   334: astore #7
    //   336: aload #7
    //   338: ifnull -> 361
    //   341: aload #7
    //   343: getstatic org/eclipse/jdt/internal/compiler/impl/Constant.NotAConstant : Lorg/eclipse/jdt/internal/compiler/impl/Constant;
    //   346: if_acmpeq -> 361
    //   349: aload #7
    //   351: invokevirtual booleanValue : ()Z
    //   354: ifne -> 361
    //   357: iconst_1
    //   358: goto -> 362
    //   361: iconst_0
    //   362: istore #14
    //   364: iload #14
    //   366: ifeq -> 372
    //   369: goto -> 397
    //   372: aload_2
    //   373: getfield position : I
    //   376: istore #14
    //   378: aload_2
    //   379: aload #12
    //   381: invokevirtual goto_ : (Lorg/eclipse/jdt/internal/compiler/codegen/BranchLabel;)V
    //   384: aload_2
    //   385: iload #14
    //   387: aload_0
    //   388: getfield valueIfTrue : Lorg/eclipse/jdt/internal/compiler/ast/Expression;
    //   391: getfield sourceEnd : I
    //   394: invokevirtual recordPositionsFrom : (II)V
    //   397: iload #10
    //   399: ifeq -> 452
    //   402: aload #11
    //   404: invokevirtual place : ()V
    //   407: aload_0
    //   408: getfield falseInitStateIndex : I
    //   411: iconst_m1
    //   412: if_icmpeq -> 433
    //   415: aload_2
    //   416: aload_1
    //   417: aload_0
    //   418: getfield falseInitStateIndex : I
    //   421: invokevirtual removeNotDefinitelyAssignedVariables : (Lorg/eclipse/jdt/internal/compiler/lookup/Scope;I)V
    //   424: aload_2
    //   425: aload_1
    //   426: aload_0
    //   427: getfield falseInitStateIndex : I
    //   430: invokevirtual addDefinitelyAssignedVariables : (Lorg/eclipse/jdt/internal/compiler/lookup/Scope;I)V
    //   433: aload_0
    //   434: getfield valueIfFalse : Lorg/eclipse/jdt/internal/compiler/ast/Expression;
    //   437: aload_1
    //   438: aload_2
    //   439: aload_3
    //   440: aload #4
    //   442: iload #5
    //   444: invokevirtual generateOptimizedBoolean : (Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;Lorg/eclipse/jdt/internal/compiler/codegen/CodeStream;Lorg/eclipse/jdt/internal/compiler/codegen/BranchLabel;Lorg/eclipse/jdt/internal/compiler/codegen/BranchLabel;Z)V
    //   447: aload #12
    //   449: invokevirtual place : ()V
    //   452: aload_0
    //   453: getfield mergedInitStateIndex : I
    //   456: iconst_m1
    //   457: if_icmpeq -> 469
    //   460: aload_2
    //   461: aload_1
    //   462: aload_0
    //   463: getfield mergedInitStateIndex : I
    //   466: invokevirtual removeNotDefinitelyAssignedVariables : (Lorg/eclipse/jdt/internal/compiler/lookup/Scope;I)V
    //   469: aload_2
    //   470: iload #6
    //   472: aload_0
    //   473: getfield sourceEnd : I
    //   476: invokevirtual recordPositionsFrom : (II)V
    //   479: return
    // Line number table:
    //   Java source line number -> byte code offset
    //   #337	-> 0
    //   #339	-> 6
    //   #340	-> 27
    //   #341	-> 44
    //   #342	-> 61
    //   #343	-> 72
    //   #345	-> 73
    //   #346	-> 82
    //   #348	-> 91
    //   #349	-> 107
    //   #348	-> 123
    //   #347	-> 128
    //   #351	-> 130
    //   #352	-> 146
    //   #351	-> 162
    //   #350	-> 167
    //   #354	-> 169
    //   #357	-> 179
    //   #358	-> 202
    //   #359	-> 206
    //   #360	-> 207
    //   #361	-> 208
    //   #362	-> 209
    //   #363	-> 220
    //   #358	-> 222
    //   #365	-> 225
    //   #366	-> 233
    //   #367	-> 234
    //   #368	-> 235
    //   #366	-> 239
    //   #369	-> 242
    //   #372	-> 251
    //   #373	-> 256
    //   #375	-> 270
    //   #378	-> 275
    //   #379	-> 280
    //   #381	-> 284
    //   #382	-> 290
    //   #383	-> 318
    //   #387	-> 326
    //   #388	-> 330
    //   #389	-> 336
    //   #390	-> 364
    //   #395	-> 372
    //   #396	-> 378
    //   #397	-> 384
    //   #403	-> 397
    //   #404	-> 402
    //   #405	-> 407
    //   #406	-> 415
    //   #407	-> 424
    //   #409	-> 433
    //   #412	-> 447
    //   #415	-> 452
    //   #416	-> 460
    //   #419	-> 469
    //   #420	-> 479
    // Local variable table:
    //   start	length	slot	name	descriptor
    //   0	480	0	this	Lorg/eclipse/jdt/internal/compiler/ast/ConditionalExpression;
    //   0	480	1	currentScope	Lorg/eclipse/jdt/internal/compiler/lookup/BlockScope;
    //   0	480	2	codeStream	Lorg/eclipse/jdt/internal/compiler/codegen/CodeStream;
    //   0	480	3	trueLabel	Lorg/eclipse/jdt/internal/compiler/codegen/BranchLabel;
    //   0	480	4	falseLabel	Lorg/eclipse/jdt/internal/compiler/codegen/BranchLabel;
    //   0	480	5	valueRequired	Z
    //   6	474	6	pc	I
    //   82	398	7	cst	Lorg/eclipse/jdt/internal/compiler/impl/Constant;
    //   91	389	8	condCst	Lorg/eclipse/jdt/internal/compiler/impl/Constant;
    //   130	350	9	needTruePart	Z
    //   169	311	10	needFalsePart	Z
    //   220	260	11	internalFalseLabel	Lorg/eclipse/jdt/internal/compiler/codegen/BranchLabel;
    //   179	301	12	endifLabel	Lorg/eclipse/jdt/internal/compiler/codegen/BranchLabel;
    //   202	278	13	needConditionValue	Z
    //   318	8	14	isValueIfTrueOptimizedTrue	Z
    //   364	8	14	isValueIfTrueOptimizedFalse	Z
    //   378	19	14	position	I
  }
  
  public int nullStatus(FlowInfo flowInfo, FlowContext flowContext) {
    if ((this.implicitConversion & 0x200) != 0)
      return 4; 
    return this.nullStatus;
  }
  
  public Constant optimizedBooleanConstant() {
    return (this.optimizedBooleanConstant == null) ? this.constant : this.optimizedBooleanConstant;
  }
  
  public StringBuffer printExpressionNoParenthesis(int indent, StringBuffer output) {
    this.condition.printExpression(indent, output).append(" ? ");
    this.valueIfTrue.printExpression(0, output).append(" : ");
    return this.valueIfFalse.printExpression(0, output);
  }
  
  public TypeBinding resolveType(BlockScope scope) {
    LookupEnvironment env = scope.environment();
    long sourceLevel = (scope.compilerOptions()).sourceLevel;
    boolean use15specifics = (sourceLevel >= 3211264L);
    this.use18specifics = (sourceLevel >= 3407872L);
    if (this.use18specifics && (
      this.expressionContext == ExpressionContext.ASSIGNMENT_CONTEXT || this.expressionContext == ExpressionContext.INVOCATION_CONTEXT)) {
      this.valueIfTrue.setExpressionContext(this.expressionContext);
      this.valueIfTrue.setExpectedType(this.expectedType);
      this.valueIfFalse.setExpressionContext(this.expressionContext);
      this.valueIfFalse.setExpectedType(this.expectedType);
    } 
    if (this.constant != Constant.NotAConstant) {
      this.constant = Constant.NotAConstant;
      TypeBinding conditionType = this.condition.resolveTypeExpecting(scope, (TypeBinding)TypeBinding.BOOLEAN);
      this.condition.computeConversion((Scope)scope, (TypeBinding)TypeBinding.BOOLEAN, conditionType);
      if (this.valueIfTrue instanceof CastExpression)
        this.valueIfTrue.bits |= 0x20; 
      this.originalValueIfTrueType = this.valueIfTrue.resolveType(scope);
      if (this.valueIfFalse instanceof CastExpression)
        this.valueIfFalse.bits |= 0x20; 
      this.originalValueIfFalseType = this.valueIfFalse.resolveType(scope);
      if (conditionType == null || this.originalValueIfTrueType == null || this.originalValueIfFalseType == null)
        return null; 
    } else {
      if (this.originalValueIfTrueType.kind() == 65540)
        this.originalValueIfTrueType = this.valueIfTrue.resolveType(scope); 
      if (this.originalValueIfFalseType.kind() == 65540)
        this.originalValueIfFalseType = this.valueIfFalse.resolveType(scope); 
      if (this.originalValueIfTrueType == null || !this.originalValueIfTrueType.isValidBinding())
        return this.resolvedType = null; 
      if (this.originalValueIfFalseType == null || !this.originalValueIfFalseType.isValidBinding())
        return this.resolvedType = null; 
    } 
    if (isPolyExpression()) {
      if (this.expectedType == null || !this.expectedType.isProperType(true))
        return (TypeBinding)new PolyTypeBinding((Expression)this); 
      return this.resolvedType = computeConversions(scope, this.expectedType) ? this.expectedType : null;
    } 
    TypeBinding valueIfTrueType = this.originalValueIfTrueType;
    TypeBinding valueIfFalseType = this.originalValueIfFalseType;
    if (use15specifics && TypeBinding.notEquals(valueIfTrueType, valueIfFalseType))
      if (valueIfTrueType.isBaseType()) {
        if (valueIfFalseType.isBaseType()) {
          if (valueIfTrueType == TypeBinding.NULL) {
            valueIfFalseType = env.computeBoxingType(valueIfFalseType);
          } else if (valueIfFalseType == TypeBinding.NULL) {
            valueIfTrueType = env.computeBoxingType(valueIfTrueType);
          } 
        } else {
          TypeBinding unboxedIfFalseType = valueIfFalseType.isBaseType() ? valueIfFalseType : env.computeBoxingType(valueIfFalseType);
          if (valueIfTrueType.isNumericType() && unboxedIfFalseType.isNumericType()) {
            valueIfFalseType = unboxedIfFalseType;
          } else if (valueIfTrueType != TypeBinding.NULL) {
            valueIfFalseType = env.computeBoxingType(valueIfFalseType);
          } 
        } 
      } else if (valueIfFalseType.isBaseType()) {
        TypeBinding unboxedIfTrueType = valueIfTrueType.isBaseType() ? valueIfTrueType : env.computeBoxingType(valueIfTrueType);
        if (unboxedIfTrueType.isNumericType() && valueIfFalseType.isNumericType()) {
          valueIfTrueType = unboxedIfTrueType;
        } else if (valueIfFalseType != TypeBinding.NULL) {
          valueIfTrueType = env.computeBoxingType(valueIfTrueType);
        } 
      } else {
        TypeBinding unboxedIfTrueType = env.computeBoxingType(valueIfTrueType);
        TypeBinding unboxedIfFalseType = env.computeBoxingType(valueIfFalseType);
        if (unboxedIfTrueType.isNumericType() && unboxedIfFalseType.isNumericType()) {
          valueIfTrueType = unboxedIfTrueType;
          valueIfFalseType = unboxedIfFalseType;
        } 
      }  
    Constant condConstant, trueConstant, falseConstant;
    if ((condConstant = this.condition.constant) != Constant.NotAConstant && (
      trueConstant = this.valueIfTrue.constant) != Constant.NotAConstant && (
      falseConstant = this.valueIfFalse.constant) != Constant.NotAConstant)
      this.constant = condConstant.booleanValue() ? trueConstant : falseConstant; 
    if (TypeBinding.equalsEquals(valueIfTrueType, valueIfFalseType)) {
      this.valueIfTrue.computeConversion((Scope)scope, valueIfTrueType, this.originalValueIfTrueType);
      this.valueIfFalse.computeConversion((Scope)scope, valueIfFalseType, this.originalValueIfFalseType);
      if (TypeBinding.equalsEquals(valueIfTrueType, (TypeBinding)TypeBinding.BOOLEAN)) {
        this.optimizedIfTrueConstant = this.valueIfTrue.optimizedBooleanConstant();
        this.optimizedIfFalseConstant = this.valueIfFalse.optimizedBooleanConstant();
        if (this.optimizedIfTrueConstant != Constant.NotAConstant && 
          this.optimizedIfFalseConstant != Constant.NotAConstant && 
          this.optimizedIfTrueConstant.booleanValue() == this.optimizedIfFalseConstant.booleanValue()) {
          this.optimizedBooleanConstant = this.optimizedIfTrueConstant;
        } else if ((condConstant = this.condition.optimizedBooleanConstant()) != Constant.NotAConstant) {
          this.optimizedBooleanConstant = condConstant.booleanValue() ? 
            this.optimizedIfTrueConstant : 
            this.optimizedIfFalseConstant;
        } 
      } 
      return this.resolvedType = NullAnnotationMatching.moreDangerousType(valueIfTrueType, valueIfFalseType);
    } 
    if (valueIfTrueType.isNumericType() && valueIfFalseType.isNumericType()) {
      if ((TypeBinding.equalsEquals(valueIfTrueType, (TypeBinding)TypeBinding.BYTE) && TypeBinding.equalsEquals(valueIfFalseType, (TypeBinding)TypeBinding.SHORT)) || (
        TypeBinding.equalsEquals(valueIfTrueType, (TypeBinding)TypeBinding.SHORT) && TypeBinding.equalsEquals(valueIfFalseType, (TypeBinding)TypeBinding.BYTE))) {
        this.valueIfTrue.computeConversion((Scope)scope, (TypeBinding)TypeBinding.SHORT, this.originalValueIfTrueType);
        this.valueIfFalse.computeConversion((Scope)scope, (TypeBinding)TypeBinding.SHORT, this.originalValueIfFalseType);
        return this.resolvedType = (TypeBinding)TypeBinding.SHORT;
      } 
      if ((TypeBinding.equalsEquals(valueIfTrueType, (TypeBinding)TypeBinding.BYTE) || TypeBinding.equalsEquals(valueIfTrueType, (TypeBinding)TypeBinding.SHORT) || TypeBinding.equalsEquals(valueIfTrueType, (TypeBinding)TypeBinding.CHAR)) && 
        TypeBinding.equalsEquals(valueIfFalseType, (TypeBinding)TypeBinding.INT) && 
        this.valueIfFalse.isConstantValueOfTypeAssignableToType(valueIfFalseType, valueIfTrueType)) {
        this.valueIfTrue.computeConversion((Scope)scope, valueIfTrueType, this.originalValueIfTrueType);
        this.valueIfFalse.computeConversion((Scope)scope, valueIfTrueType, this.originalValueIfFalseType);
        return this.resolvedType = valueIfTrueType;
      } 
      if ((TypeBinding.equalsEquals(valueIfFalseType, (TypeBinding)TypeBinding.BYTE) || 
        TypeBinding.equalsEquals(valueIfFalseType, (TypeBinding)TypeBinding.SHORT) || 
        TypeBinding.equalsEquals(valueIfFalseType, (TypeBinding)TypeBinding.CHAR)) && 
        TypeBinding.equalsEquals(valueIfTrueType, (TypeBinding)TypeBinding.INT) && 
        this.valueIfTrue.isConstantValueOfTypeAssignableToType(valueIfTrueType, valueIfFalseType)) {
        this.valueIfTrue.computeConversion((Scope)scope, valueIfFalseType, this.originalValueIfTrueType);
        this.valueIfFalse.computeConversion((Scope)scope, valueIfFalseType, this.originalValueIfFalseType);
        return this.resolvedType = valueIfFalseType;
      } 
      if (BaseTypeBinding.isNarrowing(valueIfTrueType.id, 10) && 
        BaseTypeBinding.isNarrowing(valueIfFalseType.id, 10)) {
        this.valueIfTrue.computeConversion((Scope)scope, (TypeBinding)TypeBinding.INT, this.originalValueIfTrueType);
        this.valueIfFalse.computeConversion((Scope)scope, (TypeBinding)TypeBinding.INT, this.originalValueIfFalseType);
        return this.resolvedType = (TypeBinding)TypeBinding.INT;
      } 
      if (BaseTypeBinding.isNarrowing(valueIfTrueType.id, 7) && 
        BaseTypeBinding.isNarrowing(valueIfFalseType.id, 7)) {
        this.valueIfTrue.computeConversion((Scope)scope, (TypeBinding)TypeBinding.LONG, this.originalValueIfTrueType);
        this.valueIfFalse.computeConversion((Scope)scope, (TypeBinding)TypeBinding.LONG, this.originalValueIfFalseType);
        return this.resolvedType = (TypeBinding)TypeBinding.LONG;
      } 
      if (BaseTypeBinding.isNarrowing(valueIfTrueType.id, 9) && 
        BaseTypeBinding.isNarrowing(valueIfFalseType.id, 9)) {
        this.valueIfTrue.computeConversion((Scope)scope, (TypeBinding)TypeBinding.FLOAT, this.originalValueIfTrueType);
        this.valueIfFalse.computeConversion((Scope)scope, (TypeBinding)TypeBinding.FLOAT, this.originalValueIfFalseType);
        return this.resolvedType = (TypeBinding)TypeBinding.FLOAT;
      } 
      this.valueIfTrue.computeConversion((Scope)scope, (TypeBinding)TypeBinding.DOUBLE, this.originalValueIfTrueType);
      this.valueIfFalse.computeConversion((Scope)scope, (TypeBinding)TypeBinding.DOUBLE, this.originalValueIfFalseType);
      return this.resolvedType = (TypeBinding)TypeBinding.DOUBLE;
    } 
    if (valueIfTrueType.isBaseType() && valueIfTrueType != TypeBinding.NULL)
      if (use15specifics) {
        valueIfTrueType = env.computeBoxingType(valueIfTrueType);
      } else {
        scope.problemReporter().conditionalArgumentsIncompatibleTypes(this, valueIfTrueType, valueIfFalseType);
        return null;
      }  
    if (valueIfFalseType.isBaseType() && valueIfFalseType != TypeBinding.NULL)
      if (use15specifics) {
        valueIfFalseType = env.computeBoxingType(valueIfFalseType);
      } else {
        scope.problemReporter().conditionalArgumentsIncompatibleTypes(this, valueIfTrueType, valueIfFalseType);
        return null;
      }  
    if (use15specifics) {
      TypeBinding commonType = null;
      if (valueIfTrueType == TypeBinding.NULL) {
        commonType = valueIfFalseType;
      } else if (valueIfFalseType == TypeBinding.NULL) {
        commonType = valueIfTrueType;
      } else {
        commonType = scope.lowerUpperBound(new TypeBinding[] { valueIfTrueType, valueIfFalseType });
      } 
      if (commonType != null) {
        this.valueIfTrue.computeConversion((Scope)scope, commonType, this.originalValueIfTrueType);
        this.valueIfFalse.computeConversion((Scope)scope, commonType, this.originalValueIfFalseType);
        return this.resolvedType = commonType.capture((Scope)scope, this.sourceStart, this.sourceEnd);
      } 
    } else {
      if (valueIfFalseType.isCompatibleWith(valueIfTrueType)) {
        this.valueIfTrue.computeConversion((Scope)scope, valueIfTrueType, this.originalValueIfTrueType);
        this.valueIfFalse.computeConversion((Scope)scope, valueIfTrueType, this.originalValueIfFalseType);
        return this.resolvedType = valueIfTrueType;
      } 
      if (valueIfTrueType.isCompatibleWith(valueIfFalseType)) {
        this.valueIfTrue.computeConversion((Scope)scope, valueIfFalseType, this.originalValueIfTrueType);
        this.valueIfFalse.computeConversion((Scope)scope, valueIfFalseType, this.originalValueIfFalseType);
        return this.resolvedType = valueIfFalseType;
      } 
    } 
    scope.problemReporter().conditionalArgumentsIncompatibleTypes(
        this, 
        valueIfTrueType, 
        valueIfFalseType);
    return null;
  }
  
  protected boolean computeConversions(BlockScope scope, TypeBinding targetType) {
    boolean ok = true;
    if (this.originalValueIfTrueType != null && this.originalValueIfTrueType.isValidBinding())
      if (this.valueIfTrue.isConstantValueOfTypeAssignableToType(this.originalValueIfTrueType, targetType) || 
        this.originalValueIfTrueType.isCompatibleWith(targetType)) {
        this.valueIfTrue.computeConversion((Scope)scope, targetType, this.originalValueIfTrueType);
        if (this.originalValueIfTrueType.needsUncheckedConversion(targetType))
          scope.problemReporter().unsafeTypeConversion(this.valueIfTrue, this.originalValueIfTrueType, targetType); 
        if (this.valueIfTrue instanceof CastExpression && (
          this.valueIfTrue.bits & 0x4020) == 0)
          CastExpression.checkNeedForAssignedCast(scope, targetType, (CastExpression)this.valueIfTrue); 
      } else if (isBoxingCompatible(this.originalValueIfTrueType, targetType, this.valueIfTrue, (Scope)scope)) {
        this.valueIfTrue.computeConversion((Scope)scope, targetType, this.originalValueIfTrueType);
        if (this.valueIfTrue instanceof CastExpression && (
          this.valueIfTrue.bits & 0x4020) == 0)
          CastExpression.checkNeedForAssignedCast(scope, targetType, (CastExpression)this.valueIfTrue); 
      } else {
        scope.problemReporter().typeMismatchError(this.originalValueIfTrueType, targetType, (ASTNode)this.valueIfTrue, null);
        ok = false;
      }  
    if (this.originalValueIfFalseType != null && this.originalValueIfFalseType.isValidBinding())
      if (this.valueIfFalse.isConstantValueOfTypeAssignableToType(this.originalValueIfFalseType, targetType) || 
        this.originalValueIfFalseType.isCompatibleWith(targetType)) {
        this.valueIfFalse.computeConversion((Scope)scope, targetType, this.originalValueIfFalseType);
        if (this.originalValueIfFalseType.needsUncheckedConversion(targetType))
          scope.problemReporter().unsafeTypeConversion(this.valueIfFalse, this.originalValueIfFalseType, targetType); 
        if (this.valueIfFalse instanceof CastExpression && (
          this.valueIfFalse.bits & 0x4020) == 0)
          CastExpression.checkNeedForAssignedCast(scope, targetType, (CastExpression)this.valueIfFalse); 
      } else if (isBoxingCompatible(this.originalValueIfFalseType, targetType, this.valueIfFalse, (Scope)scope)) {
        this.valueIfFalse.computeConversion((Scope)scope, targetType, this.originalValueIfFalseType);
        if (this.valueIfFalse instanceof CastExpression && (
          this.valueIfFalse.bits & 0x4020) == 0)
          CastExpression.checkNeedForAssignedCast(scope, targetType, (CastExpression)this.valueIfFalse); 
      } else {
        scope.problemReporter().typeMismatchError(this.originalValueIfFalseType, targetType, (ASTNode)this.valueIfFalse, null);
        ok = false;
      }  
    return ok;
  }
  
  public void setExpectedType(TypeBinding expectedType) {
    this.expectedType = expectedType;
  }
  
  public void setExpressionContext(ExpressionContext context) {
    this.expressionContext = context;
  }
  
  public ExpressionContext getExpressionContext() {
    return this.expressionContext;
  }
  
  public Expression[] getPolyExpressions() {
    Expression[] truePolys = this.valueIfTrue.getPolyExpressions();
    Expression[] falsePolys = this.valueIfFalse.getPolyExpressions();
    if (truePolys.length == 0)
      return falsePolys; 
    if (falsePolys.length == 0)
      return truePolys; 
    Expression[] allPolys = new Expression[truePolys.length + falsePolys.length];
    System.arraycopy(truePolys, 0, allPolys, 0, truePolys.length);
    System.arraycopy(falsePolys, 0, allPolys, truePolys.length, falsePolys.length);
    return allPolys;
  }
  
  public boolean isPertinentToApplicability(TypeBinding targetType, MethodBinding method) {
    return (this.valueIfTrue.isPertinentToApplicability(targetType, method) && 
      this.valueIfFalse.isPertinentToApplicability(targetType, method));
  }
  
  public boolean isPotentiallyCompatibleWith(TypeBinding targetType, Scope scope) {
    return (this.valueIfTrue.isPotentiallyCompatibleWith(targetType, scope) && 
      this.valueIfFalse.isPotentiallyCompatibleWith(targetType, scope));
  }
  
  public boolean isFunctionalType() {
    return !(!this.valueIfTrue.isFunctionalType() && !this.valueIfFalse.isFunctionalType());
  }
  
  public boolean isPolyExpression() throws UnsupportedOperationException {
    if (!this.use18specifics)
      return false; 
    if (this.isPolyExpression)
      return true; 
    if (this.expressionContext != ExpressionContext.ASSIGNMENT_CONTEXT && this.expressionContext != ExpressionContext.INVOCATION_CONTEXT)
      return false; 
    if (this.originalValueIfTrueType == null || this.originalValueIfFalseType == null)
      return false; 
    if (this.valueIfTrue.isPolyExpression() || this.valueIfFalse.isPolyExpression())
      return true; 
    if ((this.originalValueIfTrueType.isBaseType() || (this.originalValueIfTrueType.id >= 26 && this.originalValueIfTrueType.id <= 33)) && (
      this.originalValueIfFalseType.isBaseType() || (this.originalValueIfFalseType.id >= 26 && this.originalValueIfFalseType.id <= 33)))
      return false; 
    return this.isPolyExpression = true;
  }
  
  public boolean isCompatibleWith(TypeBinding left, Scope scope) {
    return isPolyExpression() ? ((this.valueIfTrue.isCompatibleWith(left, scope) && this.valueIfFalse.isCompatibleWith(left, scope))) : 
      super.isCompatibleWith(left, scope);
  }
  
  public boolean isBoxingCompatibleWith(TypeBinding targetType, Scope scope) {
    return isPolyExpression() ? (((this.valueIfTrue.isCompatibleWith(targetType, scope) || 
      this.valueIfTrue.isBoxingCompatibleWith(targetType, scope)) && (
      this.valueIfFalse.isCompatibleWith(targetType, scope) || 
      this.valueIfFalse.isBoxingCompatibleWith(targetType, scope)))) : 
      super.isBoxingCompatibleWith(targetType, scope);
  }
  
  public boolean sIsMoreSpecific(TypeBinding s, TypeBinding t, Scope scope) {
    if (super.sIsMoreSpecific(s, t, scope))
      return true; 
    return isPolyExpression() ? (
      (this.valueIfTrue.sIsMoreSpecific(s, t, scope) && this.valueIfFalse.sIsMoreSpecific(s, t, scope))) : false;
  }
  
  public void traverse(ASTVisitor visitor, BlockScope scope) {
    if (visitor.visit(this, scope)) {
      this.condition.traverse(visitor, scope);
      this.valueIfTrue.traverse(visitor, scope);
      this.valueIfFalse.traverse(visitor, scope);
    } 
    visitor.endVisit(this, scope);
  }
}
